﻿-- Author      : canon
-- Create Date : 7/25/2013 9:51:02 PM

local frame = CreateFrame("BUTTON", "PKTracker");
local events = {};
local genders = { "unknown", "Male", "Female" };
local playerGUID;

local classColors = {
	["Death Knight"] = "C41F3B",
	["Druid"] = "FF7D0A",
	["Hunter"] = "ABD473",
	["Mage"] = "69CCF0",
	["Monk"] = "00FF96",
	["Paladin"] = "F58CBA",
	["Priest"] = "FFFFFF",
	["Rogue"] = "FFF569",
	["Shaman"] = "0070DE",
	["Warlock"] = "9482C9",
	["Warrior"] = "C79C6E",
};

-- slightly desaturated and darkened versions of the colors above
local classFrameColors = {
	["Death Knight"] = { r = 0.122788235294118, g = 0.0386705882352941, b = 0.0529450980392157 },
	["Druid"] = { r = 0.171862745098039, g = 0.105588235294118, b = 0.0469607843137255 },
	["Hunter"] = { r = 0.139333333333333, g = 0.160235294117647, b = 0.11078431372549 },
	["Mage"] = { r = 0.102364705882353, g = 0.152835294117647, b = 0.171188235294118 },
	["Monk"] = { r = 0.0461176470588235, g = 0.176117647058824, b = 0.122588235294118 },		
	["Paladin"] = { r = 0.173243137254902, g = 0.119713725490196, b = 0.143164705882353 },
	["Priest"] = { r = .2, g = .2, b = .2},
	["Rogue"] = { r = 0.194235294117647, g = 0.189137254901961, 0.117764705882353 },
	["Shaman"] = { r = 0.0245411764705882, g = 0.0816392156862745, b = 0.137717647058824 },
	["Warlock"] = { r = 0.11456862745098, g = 0.105392156862745, b = 0.141588235294118 },
	["Warrior"] = { r = 0.146552941176471, g = 0.12463137254902, b = 0.101180392156863 },
};

local CLASS_TEX_COORDS = {
	["Warrior"]     = {0, 0.25, 0, 0.25},
	["Mage"]        = {0.25, 0.49609375, 0, 0.25},
	["Rogue"]       = {0.49609375, 0.7421875, 0, 0.25},
	["Druid"]        = {0.7421875, 0.98828125, 0, 0.25},
	["Hunter"]      = {0, 0.25, 0.25, 0.5},
	["Shaman"]      = {0.25, 0.49609375, 0.25, 0.5},
	["Priest"]      = {0.49609375, 0.7421875, 0.25, 0.5},
	["Warlock"]     = {0.7421875, 0.98828125, 0.25, 0.5},
	["Paladin"]     = {0, 0.25, 0.5, 0.75},
	["Death Knight"] = {0.25, 0.49609375, 0.5, 0.75},
	["Monk"]        = {0.496039375, 0.7421875, 0.5, 0.75},
};

local combatants = {};

local currentRecord = 0;

local combat

local leftCombat

local tracking

local function UnitGUIDIsPlayer(UnitGUID)	    
	return UnitGUID:find("Player");
end

local function nvl(a,b)
	if a ~= nil then
		return a;
	else
		return b;
	end	
end

local function iif(condition, a, b)
	
	if condition then
		return a;
	else
		return b;
	end
	
end

local function SendUnitUpdateMessage(unitInfo)
	
	local message = "UNIT_UPDATE:"
	local n = 1		
	for k,v in pairs(unitInfo) do	
		message = string.format("%s%s%s=%s", message, iif(n == 1, "", ","), k, tostring(nvl(v, "")))
		n = n + 1	
	end			
	SendAddonMessage("PKTRACKER",message,"RAID",nil);

end

local function GetUnitSummary(unit)

	local name, realm = UnitName(unit);

	return {
		GUID = UnitGUID(unit),
		Name = name,
		Realm = realm,
		Class = UnitClass(unit),
		Level = UnitLevel(unit),
		Race = UnitRace(unit),
		Sex = UnitSex(unit),		
		GuildName = GetGuildInfo(unit),
		Time = time()
	};

end

local function IsHostilePlayer(flags)

	return bit.band(flags, COMBATLOG_OBJECT_TYPE_PLAYER) > 0 and bit.band(flags, COMBATLOG_OBJECT_REACTION_HOSTILE) > 0;

end

local function WipeInfo()

	local now = time();
	
	for key, value in pairs(info) do
		
		if not combatants[key] and not value.Keep and (not value.Time or now - value.Time > 21600) then
		
			info[key] = nil;
		
		end
		
	end

end

local function UpdateKillFrame(killFrame, kill, n)		
	
	local unitInfo = nvl(info[kill.GUID],{})
		
	local name = nvl(unitInfo.Name, "Unknown");
	
	if unitInfo.Realm and not name:find("-") then
		name = string.format("%s-%s", name, unitInfo.Realm)
	end
				
	if unitInfo.Class then -- extended info		
	
		killFrame.ClassIcon:SetTexture([[Interface\Glues\CharacterCreate\UI-CharacterCreate-Classes]]);
		killFrame.ClassIcon:SetTexCoord(unpack(CLASS_TEX_COORDS[unitInfo.Class]));	
	
		killFrame.PlayerName:SetText(string.format("|cFF%s%s", classColors[unitInfo.Class], name));		
		
		if unitInfo.GuildName then
			killFrame.PlayerGuild:SetText(string.format("|cFFe9e900<%s>", unitInfo.GuildName));
			killFrame.PlayerGuild:SetAlpha(1)
		else
			killFrame.PlayerGuild:SetText("|cFFCCCCCCno guild");
			killFrame.PlayerGuild:SetAlpha(.5)
		end			
		
		killFrame.PlayerLevel:SetText(string.format("%s %s %s |cFF%s%s", iif(not kill.Level or kill.Level == -1, "??", kill.Level), genders[unitInfo.Sex], unitInfo.Race, classColors[unitInfo.Class], unitInfo.Class));
		killFrame.PlayerLevel:Show();
		killFrame.PlayerLevel:SetAlpha(1)
		
		local c = classFrameColors[unitInfo.Class];					
		
		killFrame.texture:SetTexture(c.r, c.g, c.b, .75);
		
	else -- no extended info
	
		killFrame.PlayerName:SetText(string.format("|cFFCCCCCC%s", name));	
		
		killFrame.PlayerGuild:SetText("|cFFCCCCCCunkown guild");
		killFrame.PlayerGuild:SetAlpha(.5)
		
		killFrame.PlayerLevel:SetText("|cFFCCCCCCunkown level, gender, race, & class");
		killFrame.PlayerLevel:SetAlpha(.5)
						
		killFrame.ClassIcon:SetTexture([[Interface\Icons\INV_Misc_QuestionMark]]);
		killFrame.ClassIcon:SetTexCoord(0,1,0,1);			
		
		killFrame.texture:SetTexture(0,0,0,.5);
		
	end		
	
	killFrame.KillTime:SetText(date("%m/%d/%y %I:%M%p", kill.Time));

	if kill.Location.Zone2 and kill.Location.Zone2 ~= kill.Location.Zone then
		killFrame.Zone2:SetText(kill.Location.Zone2);				
	else	
		killFrame.Zone2:SetText(string.format("%.1f, %.1f", kill.Location.X, kill.Location.Y));				
	end

	killFrame.Zone:SetText(kill.Location.Zone);		
	
	local killCount = 0

	for i = 1, getn(history) do
	
		if history[i].GUID == kill.GUID then
		
			killCount = killCount + 1;
		
		end
	
	end
	
	killFrame.TimesKilled:SetText(string.format("Total kills: %d", killCount));
	
	if kill.MyKB then
	
		killFrame.KilledBy:SetText("Killing blow: |cFF00FF00mine");
	
	else
	
		killFrame.KilledBy:SetText("Killing blow: |cFFE9E900teammate");
	
	end	

end

local function CreateKillFrame(i)	
		
	local killFrame = CreateFrame("FRAME", nil, frame);
	frame["KILLFRAME" .. i] = killFrame;
	killFrame:SetWidth(210);	
	killFrame:SetHeight(47); --37	

	local bg = killFrame:CreateTexture(nil, "BACKGROUND");
	bg:SetTexture(0,0,0,.5);
	bg:SetAllPoints(killFrame);
	killFrame.texture = bg;

	killFrame.ContentFrame = CreateFrame("FRAME",nil,killFrame);	
	killFrame.ContentFrame:SetWidth(200);	
	killFrame.ContentFrame:SetHeight(27);		
	killFrame.ContentFrame:SetPoint("TOPLEFT", killFrame, "TOPLEFT", 5, -5);		

	killFrame.ClassIcon = killFrame.ContentFrame:CreateTexture();
	killFrame.ClassIcon:SetHeight(8);
	killFrame.ClassIcon:SetWidth(8);
	killFrame.ClassIcon:SetTexture([[Interface\Glues\CharacterCreate\UI-CharacterCreate-Classes]]);
	killFrame.ClassIcon:SetPoint("TOPLEFT", killFrame.ContentFrame, "TOPLEFT", 1, 0);

	killFrame.PlayerName = killFrame.ContentFrame:CreateFontString();
	killFrame.PlayerName:SetFont("Fonts\\FRIZQT__.TTF", 9, "OUTLINE");		
	killFrame.PlayerName:SetWidth(123);
	killFrame.PlayerName:SetWordWrap(false);	
	killFrame.PlayerName:SetJustifyH("left");
	killFrame.PlayerName:SetPoint("LEFT", killFrame.ClassIcon, "RIGHT", 2, 0);
		
	killFrame.PlayerGuild = killFrame.ContentFrame:CreateFontString();
	killFrame.PlayerGuild:SetFont("Fonts\\ARIALN.TTF", 8, "OUTLINE");
	killFrame.PlayerGuild:SetPoint("LEFT", killFrame.ContentFrame, "LEFT");			
	
	killFrame.PlayerLevel = killFrame.ContentFrame:CreateFontString(); 	
	killFrame.PlayerLevel:SetPoint("BOTTOMLEFT", killFrame.ContentFrame, "BOTTOMLEFT");							
	killFrame.PlayerLevel:SetFont("Fonts\\ARIALN.TTF", 8, "OUTLINE");
				
	killFrame.KillTime = killFrame.ContentFrame:CreateFontString(); 	
	killFrame.KillTime:SetPoint("TOPRIGHT", killFrame.ContentFrame, "TOPRIGHT");							
	killFrame.KillTime:SetFont("Fonts\\ARIALN.TTF", 8, "OUTLINE");		
	
	killFrame.Zone2 = killFrame.ContentFrame:CreateFontString(); 	
	killFrame.Zone2:SetPoint("RIGHT", killFrame.ContentFrame, "RIGHT");							
	killFrame.Zone2:SetFont("Fonts\\ARIALN.TTF", 8, "OUTLINE");
	
	killFrame.Zone = killFrame.ContentFrame:CreateFontString(); 	
	killFrame.Zone:SetPoint("BOTTOMRIGHT", killFrame.ContentFrame, "BOTTOMRIGHT");							
	killFrame.Zone:SetFont("Fonts\\ARIALN.TTF", 8, "OUTLINE");	
	
	killFrame.KilledBy = killFrame.ContentFrame:CreateFontString();
	killFrame.KilledBy:SetFont("Fonts\\ARIALN.TTF", 8, "OUTLINE");
	killFrame.KilledBy:SetPoint("BOTTOMLEFT", killFrame.ContentFrame, "BOTTOMLEFT", 0, -10);			
	
	killFrame.TimesKilled = killFrame.ContentFrame:CreateFontString();
	killFrame.TimesKilled:SetFont("Fonts\\ARIALN.TTF", 8, "OUTLINE");
	killFrame.TimesKilled:SetPoint("BOTTOMRIGHT", killFrame.ContentFrame, "BOTTOMRIGHT", 0, -10);				

	return killFrame;
	
end

local function UpdateKillFrames(n, newKill)

	local killFrame, count, nHistory, nMax;
		
	nHistory = getn(history);
	nMax = nHistory - nHistory % 7 + iif(nHistory % 7 == 0, -6, 1);
			
	n = nvl(n, nHistory);

	count = 0;
		
	n = n - n % 7 + 1;
		
	if n > nMax then
	
		n = nMax;
		currentRecord = n;
	
	elseif n > 0 and nHistory > 0 then
	
		if currentRecord ~= n then
		
			currentRecord = n;
			if settings.Sound and not newKill then
				PlaySound("igAbiliityPageTurn");
			end
			
		end
	
	else
	
		n = currentRecord;
				
	end

	for i = 1, 7 do
		
		killFrame = frame["KILLFRAME" .. i];
						
		if n <= nHistory and n > 0 then
		
			UpdateKillFrame(killFrame, history[n], n)			
			killFrame.ContentFrame:Show();		
			if newKill and n == nHistory and settings.Sound then
				PlaySound("igAbilityIconDrop"); -- igPVPUpdate
			end			
			n = n + 1;
			count = count + 1;			
		else
		
			killFrame.texture:SetTexture(0,0,0,.5);
			killFrame.ContentFrame:Hide();
		
		end		
		
	end	
	
	if nHistory > 0 then		
		local nPages = math.ceil(nHistory / 7);
		frame.PageText:SetText(string.format("Page %d of %d", math.ceil(currentRecord / nHistory * nPages), nPages));
	else
		frame.PageText:SetText("No PvP Kills recorded.");
	end
		
	if currentRecord > 1 then		
		frame.Back:Enable();
	else	
		frame.Back:Disable();
	end
	
	if currentRecord < nMax and nHistory > 0 then
		frame.Next:Enable();
	else
		frame.Next:Disable();
	end
	
end

local function CreateNavButton(text, clickHandler)

	local button = CreateFrame("Button",nil,frame);
	button:SetWidth(50);
	button:SetHeight(20);
	button:SetText(text);
	button:SetNormalFontObject("GameFontNormal");
	
	local ntex = button:CreateTexture();
	ntex:SetTexture("Interface/Buttons/UI-Panel-Button-Up");
	ntex:SetTexCoord(0, 0.625, 0, 0.6875);
	ntex:SetAllPoints();
	button:SetNormalTexture(ntex);
	
	local htex = button:CreateTexture();
	htex:SetTexture("Interface/Buttons/UI-Panel-Button-Highlight");
	htex:SetTexCoord(0, 0.625, 0, 0.6875);
	htex:SetAllPoints();
	button:SetHighlightTexture(htex);
	
	local ptex = button:CreateTexture();
	ptex:SetTexture("Interface/Buttons/UI-Panel-Button-Down");
	ptex:SetTexCoord(0, 0.625, 0, 0.6875);
	ptex:SetAllPoints();
	button:SetPushedTexture(ptex);
	
	button:SetScript("OnClick", clickHandler); 
	button:SetScale(.75);
	
	return button;

end

local function CreateKillFrames()

	local anchor = frame;				
	local n = 1;

	frame:SetFrameStrata("BACKGROUND");
	frame:Show();
	frame:SetWidth(210);
	frame:SetHeight(346);	
	if settings.Scale then
		frame:SetScale(settings.Scale)
	end	
	
	frame:SetPoint(nvl(settings.point,"LEFT"), nvl(settings.relativeTo, "UIParent"), nvl(settings.relativePoint, "LEFT"), nvl(settings["x-offset"], 5), nvl(settings["y-offset"], 0));
	
	frame.PageText = frame:CreateFontString("TOP"); 	
	frame.PageText:SetPoint("TOP", frame, "TOP", 0, -4);							
	frame.PageText:SetFont("Fonts\\ARIALN.TTF", 8, "OUTLINE");	
	
	frame.Back = CreateNavButton("Older", function(self, e, ...)
		UpdateKillFrames(currentRecord - 7);
	end);
	frame.Back:SetPoint("TOPLEFT", frame, "TOPLEFT");		
	
	frame.Next = CreateNavButton("Newer", function(self, e, ...)
		UpdateKillFrames(currentRecord + 7);
	end);
	frame.Next:SetPoint("TOPRIGHT", frame, "TOPRIGHT");	
		
	for i = 1, 7 do

		local f = CreateKillFrame(i);
		if i == 1 then
			f:SetPoint("TOPLEFT", frame, "TOPLEFT", 0, -16);
		else
			f:SetPoint("TOPLEFT", anchor, "BOTTOMLEFT", 0, -1);
		end
		anchor = f;		
		
	end	
	
end

local function UpdateKillFrameUnitInfo(unitInfo)

	for i = currentRecord, math.min(currentRecord + 6, #history) do
	
		if history[i] and history[i].GUID == unitInfo.GUID then
			UpdateKillFrames(currentRecord);
			break
		end
	
	end

end

local function UpdateUnit(unitId)

	if UnitExists(unitId) then	

		local summary = GetUnitSummary(unitId);		
			
		if UnitGUIDIsPlayer(summary.GUID) and UnitIsEnemy(unitId, "player") then			
			
			if info[summary.GUID] and summary.Name == "Unknown" then
			
				summary = info[summary.GUID]
				
			else
			
				summary.Keep = info[summary.GUID] and info[summary.GUID].Keep
				
				info[summary.GUID] = summary;						
				
			end								
			
			UpdateKillFrameUnitInfo(summary)
			
			if summary.Name and summary.Name ~= "Unknown" then
				
				SendUnitUpdateMessage(summary)
				
			end
			
		end
	
	end

end

events.ADDON_LOADED = function(...)	

	if select(1, ...) == "PKTracker" then	
		
		playerGUID = UnitGUID("player");

		frame:UnregisterEvent("ADDON_LOADED");	
		
		seterrorhandler(function(msg)
			print(msg);
		end);		

		if history == nil then
			history = {};			
		end  	
		
		if info == nil then
			info = {};		
		end
		
		if settings == nil then
			settings = {
				Scale = .85				
			};
		end
		
		if settings.Sound == nil then
			settings.Sound = true;
		end
		
		WipeInfo();
		currentRecord = getn(history);			
		CreateKillFrames();
		UpdateKillFrames();
		
		if settings.Hide then
			frame:Hide();					
		end
	
	end

end

events.PLAYER_ENTER_COMBAT = function(...)
	
	combat = true
	
	frame:SetScript("OnUpdate", nil)
	
end

events.PLAYER_LEAVE_COMBAT = function(...)
    
    combat = false      
    
    leftCombat = 0
    
	frame:SetScript("OnUpdate", function(self, elapsed)		
	
		leftCombat = leftCombat + elapsed
	
		if leftCombat >= 20 then -- clear combatants 15 seconds after leaving combat
		
			frame:SetScript("OnUpdate", nil)
	
			for key in pairs(combatants) do
			
				combatants[key] = nil;
			
			end
			
		end
	 
	end);           
   
end

events.COMBAT_LOG_EVENT_UNFILTERED = function (...)
   
	local timestamp, 
	event, 
	hideCaster, 
	sourceGUID, 
	sourceName, 
	sourceFlags, 
	sourceFlags2, 
	destGUID, 
	destName, 
	destFlags, 
	destFlags2,
	spellId,
	spellName,
	spellSchool,
	amount,
	overkill = select(1, ...);		
	
	if not destFlags or not IsHostilePlayer(destFlags) then
		return	
	end	
	
	if event == "PARTY_KILL" or (event == "UNIT_DIED" and combatants[destGUID] == 3) then			
		
		combatants[destGUID] = iif(event == "PARTY_KILL", 2, nil);
						
		local x, y = GetPlayerMapPosition("player");
					
		local targetInfo = info[destGUID]
		
		if not targetInfo then
		
			targetInfo = {
				GUID = destGUID,
				Name = destName
			}
			
			info[destGUID] = targetInfo
		
		end				
		
		local kill = { 
			GUID = destGUID,
			Level = targetInfo.Level,
			Time = timestamp,
			Location = {
				Zone = GetZoneText("player"),
				Zone2 = GetMinimapZoneText(),
				X = x * 100,
				Y = y * 100
			},
			MyKB = UnitGUID("player") == sourceGUID
		};							
				
		targetInfo.Keep = true		
		table.insert(history, kill);		
		UpdateKillFrames(getn(history), true);			
	
	elseif string.find(event,"_DAMAGE") then
	
		if combatants[destGUID] and combatants[destGUID] ~= 2 and overkill and overkill > 0 then
		
			-- Allow UNIT_DIED to pick up the kill if it wasn't a PARTY_KILL			
			combatants[destGUID] = 3
			
		elseif sourceGUID == playerGUID and sourceGUID ~= destGUID and not combatants[destGUID] then
		
			-- Add unit GUID to combatants
			combatants[destGUID] = 1
		
		end
	
	elseif string.find(event,"_INSTAKILL") then
	
		-- stupid purgatory...
	
		if combatants[destGUID] and combatants[destGUID] ~= 2 then

			-- Allow UNIT_DIED to pick up the kill if it wasn't a PARTY_KILL			
			combatants[destGUID] = 3		
						
		end
			
	end
   
end

events.UNIT_TARGET = function (...)
   
	local target = select(1, ...) .. "target";			

	UpdateUnit(target)	
   
end

events.UNIT_NAME_UPDATE = function(unitId)

	UpdateUnit(unitId)

end

local function OnZoneSwapped()
	local pvpType, isFFA, faction = GetZonePVPInfo();

	if pvpType == "combat" or IsInInstance() then
		if tracking then
			print("Pausing PK tracking...")
		end
		tracking = false
		frame:Hide()
		frame:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED");
		frame:UnregisterEvent("UNIT_TARGET");
		frame:UnregisterEvent("PLAYER_ENTER_COMBAT");
		frame:UnregisterEvent("PLAYER_LEAVE_COMBAT");		
	else		
		if not tracking then
			print("Resuming PK tracking...")
		end
		tracking = true
		if settings and not settings.Hide then
			frame:Show()
		end
		frame:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED");
		frame:RegisterEvent("UNIT_TARGET");
		frame:RegisterEvent("PLAYER_ENTER_COMBAT");
		frame:RegisterEvent("PLAYER_LEAVE_COMBAT");
	end
end

events.PLAYER_ENTERING_WORLD = function (...)
   OnZoneSwapped(); 
end

events.ZONE_CHANGED_NEW_AREA = function (...)
   OnZoneSwapped();
end

events.custom = {}

events.custom.UNIT_UPDATE = function (msg)
	local unitInfo = {}
	local parts
	
	for pair in msg:gmatch("[^,]+") do
		parts = {}	
		for part in pair:gmatch("[^=]+") do
			table.insert(parts, part)
		end
		if parts[2] then
			unitInfo[parts[1]] = parts[2]		
		end
	end	
	
	if unitInfo.Keep then
		unitInfo.Keep = unitInfo.Keep == "true"
	end
	
	if unitInfo.Level then
		unitInfo.Level = tonumber(unitInfo.Level, 10)
	end
	
	if unitInfo.Sex then
		unitInfo.Sex = tonumber(unitInfo.Sex, 10)	
	end

	if unitInfo.Time then
		unitInfo.Time = tonumber(unitInfo.Time, 10)	
	end			
	
	if info[unitInfo.GUID] then
	
		for k,v in pairs(unitInfo) do
			info[unitInfo.GUID][k] = v;		
		end

	else
	
		info[unitInfo.GUID] = unitInfo
		
	end	
	
	UpdateKillFrameUnitInfo(unitInfo)	
	
end

events.CHAT_MSG_ADDON = function(prefix, message, type, sender)
	
	if prefix ~= "PKTRACKER" then
		return
	end
	
	local eventName = message:match("^[A-Z_]+")
	
	local handler = events.custom[eventName];
	
	if handler then
		handler(message:sub(#eventName + 2))	
	end
	
end

frame:RegisterEvent("ADDON_LOADED");
frame:RegisterEvent("CHAT_MSG_ADDON");
frame:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED");
frame:RegisterEvent("PLAYER_ENTERING_WORLD");
frame:RegisterEvent("PLAYER_LEAVE_COMBAT");
frame:RegisterEvent("PLAYER_ENTER_COMBAT");
frame:RegisterEvent("UNIT_NAME_UPDATE");
frame:RegisterEvent("UNIT_TARGET");
frame:RegisterEvent("ZONE_CHANGED_NEW_AREA");

frame:SetScript("OnEvent", function(self, event, ...)
 events[event](...);
end);

RegisterAddonMessagePrefix("PKTRACKER");

SLASH_PKTtracker1,SLASH_PKTtracker2 = "/pkt", "/pktracker";
SlashCmdList.PKTtracker = function(msg, editbox)
	if msg == "hide" then
		settings.Hide = true;
		frame:Hide();
	elseif msg == "info" then
		print(string.format("%d total kills.", getn(history)));
	elseif msg == "resetdata" then		
		for key in pairs(history) do			
			history[key] = nil;			
		end		
		for key in pairs(info) do			
			info[key] = nil;			
		end				
		for key in pairs(combatants) do			
			combatants[key] = nil;			
		end				
		UpdateKillFrames();
		print("Data reset.");	
	elseif msg == "resetposition" then				
		settings.point = "LEFT"
		settings.relativeTo = "UIParent"			
		settings.relativePoint = "LEFT"
		settings["x-offset"] = 5;						
		settings["y-offset"] = 0;						
		frame:SetPoint("LEFT","UIParent","LEFT",5,0);
	elseif msg == "show" then
		settings.Hide = nil;
		frame:Show();
	elseif msg == "sound" then
		settings.Sound = not settings.Sound;
		print(iif(settings.Sound, "Sound enabled.", "Sound disabled."));				
	elseif msg:find("scale") then
	    local n = msg:match("%d*%.?%d+")	    
	    if n then
			local scale = tonumber(n,10)
			print(n,scale)
			frame:SetScale(scale);		
			settings.Scale = scale;
		end	
	elseif msg:find("x-offset") then
	    local n = msg:match("%d*%.?%d+")	    
	    if n then
			settings["x-offset"] = tonumber(n,10);						
			frame:SetPoint(nvl(settings.point,"LEFT"), nvl(settings.relativeTo, "UIParent"), nvl(settings.relativePoint, "LEFT"), n, nvl(settings["y-offset"], 0));
		end						
	elseif msg:find("y-offset") then
	    local n = msg:match("%d*%.?%d+")	    
	    if n then
			settings["y-offset"] = tonumber(n,10);			
			frame:SetPoint(nvl(settings.point,"LEFT"), nvl(settings.relativeTo, "UIParent"), nvl(settings.relativePoint, "LEFT"), nvl(settings["x-offset"], 5), n);
		end				
	elseif msg:find("unlock") then	
		frame:EnableMouse(true)
		frame:SetMovable(true)
		frame:RegisterForDrag("LeftButton");
		frame:SetScript("OnDragStart", function()
			frame:StartMoving();
		end);		
		frame:SetScript("OnDragStop", function()
			frame:StopMovingOrSizing();			
			settings.point, settings.relativeTo, settings.relativePoint, settings["x-offset"], settings["y-offset"] = frame:GetPoint()
		end);	
		print("PKTracker unlocked; drag to move.");
	elseif msg:find("lock") then											
		frame:EnableMouse(false)
		frame:SetMovable(false)
		frame:RegisterForDrag();
		frame:SetScript("OnDragStart", nil);		
		frame:SetScript("OnDragStop", nil);	
		print("PKTracker locked");
	else
		print("PKTracker commands:\n/pkt hide\n/pkt info\n/pkt resetdata\n/pkt resetposition\n/pkt show\n/pkt sound\n/pkt lock\n/pkt unlock\n/pkt scale .85\n/pkt x-offset 5\n/pkt y-offset 100");
	end
end